function [ int_mat ] = m_gen_PSF_num_int_mat( int_method, sampling_number_x, sampling_number_y )
% This function calculates the matrices for numerical integration.

% calculating matrices for a certain type of numerical integration method
switch int_method
    % choosing method
    case  'custom'	% my own method
        constants_x=zeros(sampling_number_x,1);   % for "x" integration
        constants_y=zeros(1,sampling_number_y);   % for "y" integration
        % going through "x" and "y" dimensions
        for idx=1:2
            if idx==1   % in case of "x" dimension
               sampling_number=sampling_number_x;
            elseif idx==2   % in case of "y" dimension
                if sampling_number_y==sampling_number_x;    % if the two dimension have the same number of sampling points
                    constants_y=constants_x';
                    break;       % skip calculting for the "y" dimension
                else
                    sampling_number=sampling_number_y;
                end
            end
            % calculating the constants for the numerical integration matrix
            custom_row=[];
            if sampling_number>=6
                custom_row=ones(1,sampling_number);
                custom_row(1,1)=9/24;
                custom_row(1,2)=28/24;
                custom_row(1,3)=23/24;
                custom_row(1,end-2)=23/24;
                custom_row(1,end-1)=28/24;
                custom_row(1,end)=9/24;
            elseif sampling_number==5
                custom_row(1,1)=9/24;
                custom_row(1,2)=28/24;
                custom_row(1,3)=22/24;
                custom_row(1,4)=28/24;
                custom_row(1,5)=9/24;
            elseif sampling_number==4
                custom_row(1,1)=9/24;
                custom_row(1,2)=27/24;
                custom_row(1,3)=27/24;
                custom_row(1,4)=9/24;
            elseif sampling_number==3
                custom_row(1,1)=1/3;
                custom_row(1,2)=4/3;
                custom_row(1,3)=1/3;
            elseif sampling_number==2
                custom_row(1,1)=1/2;
                custom_row(1,2)=1/2;
            elseif sampling_number==1
                error('Something is wrong in "m_gen_PSF_image_sampling_points.m" or with the "pixel_sampling_fill_bool" variable.')
            end
            if idx==1
                constants_x=custom_row';
            elseif idx==2
                constants_y=custom_row;
            end
        end
    case 'Simpson'  % Simpson's rule, should not be used for pixelization
        if rem(sampling_number_x,2)==0 || rem(sampling_number_y,2)==0
           error('Simpson''s rule requires odd number of sampling points.') 
        end
        constants_x=zeros(sampling_number_x,1);   % for "x" integration
        constants_y=zeros(1,sampling_number_y);   % for "y" integration
        % going through "x" and "y" dimensions
        for idx=1:2
            if idx==1   % in case of "x" dimension
               sampling_number=sampling_number_x;
            elseif idx==2   % in case of "y" dimension
                if sampling_number_y==sampling_number_x;    % if the two dimension have the same number of sampling points
                    constants_y=constants_x';
                    break;       % skip calculting for the "y" dimension
                else
                    sampling_number=sampling_number_y;
                end
            end
            % calculating the constants for the numerical integration matrix
            simpson_row=[];
            if sampling_number>=3
                simpson_row=2/3*ones(1,sampling_number);
                simpson_row(1,2:2:end)=4/3;
                simpson_row(1,1)=1/3;
                simpson_row(1,end)=1/3;
            elseif sampling_number==2
                simpson_row(1,1)=1/2;
                simpson_row(1,2)=1/2;
            elseif sampling_number==1
                error('Something is wrong in "m_gen_PSF_image_sampling_points.m" or with the "pixel_sampling_fill_bool" variable.')
            end
            if idx==1
                constants_x=simpson_row';
            elseif idx==2
                constants_y=simpson_row;
            end
        end
    otherwise
        error('Something is wrong with the "integration method" during the call of "m_gen_PSF_num_int_mat.m"')
end

int_x_mat=repmat(constants_x,1,sampling_number_y);
int_y_mat=repmat(constants_y,sampling_number_x,1);

int_mat=int_x_mat.*int_y_mat;

end

