function [ PSF_image ] = m_gen_PSF_pixelise( params, PSF, pix_x, pix_y, idx_psf_sampling_x ,idx_psf_sampling_y )
% This function sums the values at the sampled points on the image plane
% belonging to the individual pixels thus performs a simple and fast
% numerical integration

% PARENT: m_gen_PSF_numerical_calc
% CHILD: -

persistent int_mat_pers calculated_int_matrices

pixel_sampling_fill_bool=params.numcalc.pixel_sampling_fill_bool;
polarization_bool=params.meas.polarization_bool;

if ~polarization_bool
    PSF_image{1}=zeros(length(pix_x),length(pix_y));     % array for storing the pixelated image of the PSF
elseif ~polarization_bool
    PSF_image{1}=zeros(length(pix_x),length(pix_y));     % array for storing the pixelated image of the PSF
    PSF_image{2}=zeros(length(pix_x),length(pix_y));     % array for storing the pixelated image of the PSF
end

idx_pix_loc_x=0;
for idx_pix_x=pix_x
    idx_pix_loc_x=idx_pix_loc_x+1;
    idx_pix_loc_y=0;
    for idx_pix_y=pix_y
        idx_pix_loc_y=idx_pix_loc_y+1;
        if ~pixel_sampling_fill_bool    % use Euler method for numerical integration
            for idx_pol=1:length(PSF)
                PSF_image{idx_pol}(idx_pix_loc_x,idx_pix_loc_y)=sum(sum(PSF{idx_pol}(idx_psf_sampling_x{idx_pix_loc_x},idx_psf_sampling_y{idx_pix_loc_y})));  % summing the sampled values within the given pixel
            end
        else    % use other "higher" order method for numerical integration
            sampling_number_x=length(idx_psf_sampling_x{idx_pix_loc_x});
            sampling_number_y=length(idx_psf_sampling_y{idx_pix_loc_y});
            int_mat_idx=find(calculated_int_matrices==sampling_number_x+1i*sampling_number_y,1);
            if ~isempty(int_mat_idx)
                int_mat=int_mat_pers{int_mat_idx};
            else
                int_mat_pers{end+1}=m_gen_PSF_num_int_mat( 'custom', sampling_number_x, sampling_number_y );
                calculated_int_matrices=[calculated_int_matrices,sampling_number_x+1i*sampling_number_y];
                int_mat=int_mat_pers{end};
            end
            for idx_pol=1:length(PSF)
                PSF_image{idx_pol}(idx_pix_loc_x,idx_pix_loc_y)=sum(sum(PSF{idx_pol}(idx_psf_sampling_x{idx_pix_loc_x},idx_psf_sampling_y{idx_pix_loc_y}).*int_mat));  % summing the sampled values within the given pixel
            end
        end
    end
end

end

