function m_gen_seq_save(varargin)

switch length(varargin)
    case 3
        seq=varargin{1};
        file_name=varargin{2};
        polarization_bool=varargin{3};
    otherwise
        error('Invalid input.')
end

[ output_path , output_name , ext]=fileparts(file_name);
    
%file_name2=fullfile(output_path,[output_name,'.dat']);
    

input_format=isstruct(seq);
if input_format
    %Nofframe=length(seq);
    Nofframe=size(seq.frame{1},3);
else
    [ ~ , ~ , Nofframe]=size(seq);
end

% saving the molecule positions the frame indices into the ".dat" file
%dlmwrite (file_name2, seq.output_params', 'delimiter', '\t');
%for ind_im2=1:Nofframe
%    dlmwrite (file_name2,[ones(size(seq.molpos(ind_im2),2),1)*ind_im2,seq.molpos(ind_im2)'], '-append','delimiter', '\t');
%end

% saving the image stack and the data file (two stacks in case of polarization sensitive measurement)
for idx_pol=1:length(seq.frame)
    
    % generating the name of the data file
    if ~polarization_bool
        data_file_name=fullfile(output_path,[output_name,'.dat']);
    elseif idx_pol==1
        data_file_name=fullfile(output_path,[output_name,'_polX','.dat']);
    elseif idx_pol==2
        data_file_name=fullfile(output_path,[output_name,'_polY','.dat']);
    else
        error('Some error occured during data saving.')
    end
    
    % saving the data file
    if exist(data_file_name,'file')
        delete(data_file_name);
    end
    
    fileID = fopen(data_file_name,'w');
    fprintf(fileID,'%s\n', seq.output_names );
    dlmwrite (data_file_name, seq.output_params{idx_pol}, 'delimiter', '\t', '-append');
    fclose(fileID);
    
    % generating the name of the image stack
    if ~polarization_bool
        image_stack_name=file_name;
    elseif idx_pol==1
        image_stack_name=fullfile(output_path,[[output_name,'_polX'],ext]);
    elseif idx_pol==2
        image_stack_name=fullfile(output_path,[[output_name,'_polY'],ext]);
    else
        error('Some error occured during image saving.')
    end

    if exist(image_stack_name,'file')
        delete(image_stack_name);
    end
    
    % saving the image stack
    switch ext(2:end)
        case 'tif'
            if input_format
                options.overwrite=true;
                options.compress='lzw';
                %options.compress='adobe';
                options.color=false;
                options.message=false;
                
                size_x=size(fliplr(seq.frame{idx_pol}(:,:,1))',1);
                size_y=size(fliplr(seq.frame{idx_pol}(:,:,1))',2);
                output_image=uint16(zeros(size_x,size_y,Nofframe));
                for ind_im2=1:Nofframe
                    output_image(:,:,ind_im2)=uint16(fliplr(seq.frame{idx_pol}(:,:,ind_im2))');
                end
                
                % using an extarnal function, should be much faster than
                % MATLAB's "imwrite" for stacks containing large number of
                % frames
                % taken from: https://www.mathworks.com/matlabcentral/fileexchange/35684-save-and-load-a-multiframe-tiff-image
                saveastiff(output_image, image_stack_name, options);
                
                % for cycle is needed beacuse "imwrite" can only write out individual frames
                %for ind_im2=1:Nofframe
                %    imwrite(uint16(fliplr(seq.frame{idx_pol}(:,:,ind_im2))'),image_stack_name,'Compression','none','WriteMode', 'append');
                %    %if isempty(dye_written_bool{idx_dye})
                %    %    dlmwrite (file_name2,[ones(size(seq.molpos(ind_im2),2),1)*ind_im2,seq.molpos(ind_im2)'], '-append','delimiter', '\t');
                %    %end
                %end
            else
                % what is it useful for?
                % is this part oudated?
                
                
                options.overwrite=true;
                options.compress='lzw';
                %options.compress='adobe';
                options.color=false;
                options.message=false;
                
                size_x=size(fliplr(seq(:,:,1))',1);
                size_y=size(fliplr(seq(:,:,1))',2);
                output_image=uint16(zeros(size_x,size_y,Nofframe));
                for ind_im2=1:Nofframe
                    output_image(:,:,ind_im2)=uint16(fliplr(seq(:,:,ind_im2))');
                end
                
                saveastiff(output_image, image_stack_name, options);
                
                %for ind_im2=1:Nofframe
                %    imwrite(uint16(fliplr(seq(:,:,ind_im2))'),image_stack_name,'Compression','none','WriteMode', 'append');
                %    %if isempty(dye_written_bool{idx_dye})
                %    %    dlmwrite (file_name2,[ones(size(seq.molpos(ind_im2),2),1)*ind_im2,seq.molpos(ind_im2)'], '-append','delimiter', '\t');
                %    %end
                %end
            end
        case 'raw'
            fileID=fopen(image_stack_name,'w');
            % Transpose is necessary to keep columnwise order in the binary data as in tif.
            if input_format
                for ind_im2=1:Nofframe
                    fwrite(fileID, uint16(fliplr(seq.frame{idx_pol}(:,:,ind_im2))),'uint16',0,'l');
                    % arguments:
                    % 'uint16': write out as 8-bit unsigned integers
                    % 0: do not skip any bytes before writing each value
                    % '1': Little-endian ordering
                    
                    % When opening raw files with ImageJ set the correct
                    % frame size, the number of frames, "16-bit Unsigned"
                    % image type and check "Little-endian byte order"
                    
                    
                    
                    
                    %fwrite(fileID, uint16(seq.frame{idx_pol}(:,:,ind_im2)'),'uint16',0,'l');
                    %if isempty(dye_written_bool{idx_dye,idx_pol})
                    %    dlmwrite (file_name2,[ones(size(seq.molpos(ind_im2),2),1)*ind_im2,seq.molpos(ind_im2)'], '-append','delimiter', '\t');
                    %end

                end
            else
                % what is it useful for?
                % is this part oudated?
                
                fwrite(fileID, permute(uint16(seq),[2 1 3]),'uint16',0,'l');
                %if isempty(dye_written_bool{idx_dye,idx_pol})
                %    dlmwrite (file_name2,[ones(size(seq.molpos(1),2),1),seq.molpos(1)'], '-append','delimiter', '\t');
                %end

            end
            fclose(fileID);
        otherwise
            error('Invalid file type.')
    end

end

end