function [ vesicle_coords ] = m_gen_struct_vesicles( params )
% This function generates the center positions of vesicles. In case of
% user given coordinates it simply places the given coordinates in an
% array. Otherwise randomly places the vesicle centers in the given area.

% PARENT: m_gen_struct.m
% CHILDS: -

% if fixed coordinates were chosen
if isfield(params.sample,'ves_fix_N')
    vesicle_coords.pos=[...
        params.sample.ves_fix_x;...
        params.sample.ves_fix_y;...
        params.sample.ves_fix_z];
    
    R_ves=params.sample.ves_fix_rad;    % radius of the individual vesicules
    vesicle_coords.rad=R_ves;
    
    ves_N=size(params.sample.ves_fix_x,2);  % number of vesicles
    vesicle_coords.ves_N=ves_N;
    
    % rotation angles of the ellipsoid
    %theta_rot_ellipsoids=acos(1-2*rand(N_ves,1));     % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.theta=zeros(ves_N,1);      % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.fi=2*pi*rand(ves_N,1);  % angle for the ellipsoid's rotation around the "z" axis
    %fi_rot_ellipsoids=zeros(N_ves,1);  % angle for the ellipsoid's rotation around the "z" axis
    
    
% if random coordinates were chosen
elseif isfield(params.sample,'ves_N_ofves')
    ves_N=params.sample.ves_N_ofves;  % number of vesicles
    ves_dim=params.sample.ves_dim;      % "x" and "y" extension of area in which the vesicles are distributed
    ves_zmax=params.sample.ves_zmax;    % "z" extension of area in which the vesicles are distributed
    
    vesicle_coords.ves_N=ves_N;
    
    vesicle_coords.pos=[...
        (rand(1,ves_N)-0.5)*ves_dim;...
        (rand(1,ves_N)-0.5)*ves_dim;...
        (rand(1,ves_N)-0.5)*ves_zmax];  % 3x(ves_fix_N/ves_N_ofves) array containing coordinates of vesicles
    
    R_mean=repmat(params.sample.ves_Rad_ofves, 1, ves_N);   % mean radius of vesicles
    R_sig=repmat(params.sample.ves_Sig_ofves, 3, ves_N);    % sigma of radius of vesicles
    R_ves=R_mean+R_sig.*randn(3, ves_N);                    % radius of individual vesicles, following normal distribution
    vesicle_coords.rad=R_ves;
    
    % rotation angles of the ellipsoid
    %theta_rot_ellipsoids=acos(1-2*rand(N_ves,1));     % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.theta=zeros(ves_N,1);      % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.fi=2*pi*rand(ves_N,1);  % angle for the ellipsoid's rotation around the "z" axis
    %fi_rot_ellipsoids=zeros(N_ves,1);  % angle for the ellipsoid's rotation around the "z" axis
    
else
    error('Error in vesicles coordinates');
end

end

