function output=m_gen_traj_base_dye(t_max, mol_pos, mol_orient, dye_param, flag)
% This function creates temporal trajectories for the individual dye
% molecules.
% Each dye molecule has 3 possible states, active, passive and bleached
% state. From the active state the molecule can only get into the passive
% state. From the passive state the molecule can get to the active or the
% bleached state. Once the molecule had got to the bleached state, stays
% there. Each transition has a characteristic rate time, the time of
% transition has exponential distribution.
% Each molecule sarts in either active or passive state with a probability
% determined by the characteristic rate times. The function runs until the
% dye molecule's "elapsed" time gets longer than the measurement time or it
% had got into bleached state. If the molecule is in the OFF state, it
% decide whether the next state will be ON or bleached state. If the
% molecule's next state will be bleached state, it rewrites the current OFF
% state to bleached state and exits the actual molecule. If the next state
% will be active state, calculates the life spam of the current pasive
% state. If the current state is active state, it calculates the life spam
% of the state, and the next state will be passive state. Then adds the
% life spam of the current state to the molecules "elapsed" time, and jumps
% to the next state. The trajectories of the dye molecules contains when it
% had became bleached, number of times it has got into the active state
% during the measurement,the different states the molecule had got into,
% the starting time of the states and the position at the beginning(?) the
% of different states. The positions of the dye molecule during the
% different states will be the same, the initial position, the molecule
% does not move.

%VARIABLES
% idx_mol: index of the actual dye molecule
% idx_state: index of the actual state of the actual molecule
% trajs(idx_mol).state(idx_state)=0: passive state
% trajs(idx_mol).state(idx_state)=-1: bleached state
% trajs(idx_mol).state(idx_state)=state_value: active state
% trajs(idx_mol).Nofactive: number of active states, number of times the dye molecule got into the active state
% trajs(idx_mol).pos: position of the mulecule during the different states, dos not change
% trajs(idx_mol).t(idx_state): starting time of the state
% trajs(idx_mol).bleach: contains the beginning time of the OFF state from which the actual dye molecule got into the bleached state
% state_value: a positive value, used for active states
% ttemp: duration of the actual active or passive state
% dye_param.tauon: charactersitic time of ON->OFF transition
% dye_param.tauoff: characteristic time of OFF->ON transition
% dye_param.taub: characteristic time of OFF->BLEACH transition
% t_max: time of the measurement

N_mol=size(mol_pos.init_pos,2);
%if flag~=0
%    fields_to_copy=fields(mol_pos);
%    for qwe=1:length(fields_to_copy)
%        if isequal(fields_to_copy{qwe},'init_pos')
%            fields_to_copy(qwe)=[];
%            break
%        end
%    end
%end

% memory allocation and initialization
trajs(1:N_mol)=struct(...
    'bleach',0,...
    'Nofactive',0,...
    't',0,...
    'pos',zeros(3,1)...
    );
for idx_mol=1:N_mol
    idx_state=1;
    %state_value=dye_param.N_ofphotonpersec;
    state_value=1;
    if (dye_param.tauon)/(dye_param.tauon+dye_param.tauoff)<rand(1)
        trajs(idx_mol).state=0;
    else
        trajs(idx_mol).state=state_value;
        trajs(idx_mol).pos=mol_pos.init_pos(:,idx_mol);
    end
    while trajs(idx_mol).t(idx_state)<t_max
        if trajs(idx_mol).state(idx_state)==0
            if (dye_param.taub)/(dye_param.taub+dye_param.tauoff)<rand(1)
                trajs(idx_mol).state(idx_state)=-1;
                trajs(idx_mol).bleach=trajs(idx_mol).t(idx_state);
                break
            else
                ttemp=-dye_param.tauoff*log(1-rand(1));
                trajs(idx_mol).state(idx_state+1)=state_value;
            end
        else
            ttemp=-dye_param.tauon*log(1-rand(1));
            trajs(idx_mol).state(idx_state+1)=0;
        end
        trajs(idx_mol).t(idx_state+1)=trajs(idx_mol).t(idx_state)+ttemp;
        idx_state=idx_state+1;
    end
    trajs(idx_mol).pos=repmat(mol_pos.init_pos(:,idx_mol),1,length(trajs(idx_mol).state));
    trajs(idx_mol).orient=repmat(mol_orient.init_orient(:,idx_mol),1,length(trajs(idx_mol).state));
    trajs(idx_mol).Nofactive=sum(trajs(idx_mol).state>0);
    
    %if flag~=0
    %    for temp=fields_to_copy
    %        eval(['trajs(idx_mol).',char(temp),'=mol_pos(idx_mol).',char(temp),';']);
    %    end
    %end
end

output.t_max=t_max;
output.ts=trajs;

end