global m_gen_path
output_dir=[m_gen_path,filesep,'images'];

% output_file_name=[output_dir,filesep,'test.raw'];
output_file_name2=[output_dir,filesep,'test.tif'];

tic
rng('default')

% Sample parameter settings
seq_params=[];

Size_ofactarea=20*160; %size of active area, where the vesicles are located

Zmax=258;% [nm] the maximum distance from the focal plane of the vesicle
N_ofves=10; % N of vesicles in a frame
Rad_ofves=39.5; % average radius of vesicles in nm
Sig_ofves=5.1; %Sigma of radius of ves in nm

seq_params.sample.name='vesicle';
seq_params.sample.link_rad=8; % length of the linker in nm
% N of molecules, x, y, z, coordinates, the Radius of a vesicle.
seq_params.sample.vesicle_params=[...
    randi([150 300],N_ofves,1),...
    22*160+m_gen_random('uniform',zeros(N_ofves,1),Size_ofactarea.*ones(N_ofves,1)),...
    22*160+m_gen_random('uniform',zeros(N_ofves,1),Size_ofactarea.*ones(N_ofves,1)),...
    m_gen_random('uniform',-Zmax.*ones(N_ofves,1),Zmax.*ones(N_ofves,1)),...
    Rad_ofves+Sig_ofves.*randn(N_ofves,1)];


% Dye parameter settings
seq_params.dye.name='base_dye';
seq_params.dye.tauon=0.05; % caracteristic time on state
seq_params.dye.tauoff=50; % caracteristic time off state
seq_params.dye.taub=500; % caracteristic time bleach state
seq_params.dye.lambda=647; % wavelength
seq_params.dye.N_ofphotonpersec=500000; % photonfluxus emitted by a molecule

% Acqusition / Camera settings
seq_params.aq.frame_size=64;% The frame size NxN.
seq_params.aq.frame_N=1000;% The number of frames.
seq_params.aq.frame_rate=50;% frame/s
seq_params.aq.frame_fill_factor=1;% the fraction of the exposure time used to collect photons.
seq_params.aq.pixel_size=160; %pixel size in nm
seq_params.aq.bg_level=200; %background level
seq_params.psf.name='gauss'; % type of used PSF
seq_params.psf.size=200; %width of PSF
seq_params.psf.n=1.33;  %refractive index of medium where the vesicules are located

traj_t_max=seq_params.aq.frame_N/seq_params.aq.frame_rate;


%%
tic
molecule_pos=m_gen_mol_coord(...
    seq_params.sample.name,...
    seq_params.sample,...
    1);
toc
trajs=m_gen_traj_gen(...
    seq_params.dye.name,...
    traj_t_max,...
    molecule_pos,...
    seq_params.dye,...
    1);
toc
seq=m_gen_trajs_to_seq(...
    trajs,...
    seq_params.aq,...
    seq_params.psf,...
    seq_params.dye);
toc
seq_n=m_gen_noise_addition(...
    seq,...
    seq_params.aq);
toc
% m_gen_seq_save(seq_n,output_file_name);
% toc
m_gen_seq_save(seq_n,output_file_name2);
toc

%%
temp=zeros(1,length(seq));
for qwe=1:length(seq)
    temp(qwe)=sum(seq(qwe).frame(:));
end
figure
plot(temp)

%%
temp_init=[molecule_pos.init_pos];

figure
scatter3(temp_init(1,:),...
    temp_init(2,:),...
    temp_init(3,:),'.')

set(gca,'DataAspectRatio',[1 1 1])
%%
figure
hist(temp)
%%
figure
temp=zeros(size(seq(1).frame));
for qwe=1:length(seq)
    temp=temp+seq(qwe).frame;
end
imagesc(temp,[0,max([temp(:);1])])
axis equal;
axis tight
% axis off
colormap gray


%%
figure
for qwe=1:length(seq)
    maxI=max([seq(qwe).frame(:);1]);
    imagesc(seq(qwe).frame,[0,maxI])
    axis equal; axis off
    title(num2str(qwe))
    drawnow
    pause(0.03);
    colormap gray
end






